@@ -7,6 +7,7 @@ package main
 
 import (
 	"context"
+	"crypto/fips140"
 	"encoding/json"
 	"fmt"
 	"log"
@@ -14,6 +15,7 @@ import (
 	"os"
 	"path/filepath"
 	"runtime"
+	"runtime/pprof"
 	"strings"
 	"time"
 
@@ -40,6 +42,8 @@ const (
 
 	// The parent process will create a file to collect crash logs
 	envTmpLogPath = "TF_TEMP_LOG_PATH"
+
+	EnvCPUProfile = "TOFU_CPU_PROFILE"
 )
 
 // ui wraps the primary output cli.Ui, and redirects Warn calls to Output
@@ -68,6 +72,29 @@ func main() {
 func realMain() int {
 	defer logging.PanicHandler()
 
+	// Create a go CPU profile if requested
+	// This is more intense and potentially disruptive compared to OpenTelementry tracing and does not integrate with providers
+	// It does however provide a deeper window (with more noise) into performance bottlenecks that are identified by a user
+	// or with OpenTelemetry tracing
+	if cpuProfile := os.Getenv(EnvCPUProfile); cpuProfile != "" {
+		cpuProfileOut, err := os.Create(cpuProfile)
+		if err != nil {
+			Ui.Error(fmt.Sprintf("Could not open cpu profile output: %s", err))
+			return 1
+		}
+		defer func() {
+			err := cpuProfileOut.Close()
+			if err != nil {
+				Ui.Error(fmt.Sprintf("Could not close cpu profile: %s", err))
+			}
+		}()
+		if err := pprof.StartCPUProfile(cpuProfileOut); err != nil {
+			Ui.Error(fmt.Sprintf("Could not start cpu profile: %s", err))
+			return 1
+		}
+		defer pprof.StopCPUProfile()
+	}
+
 	ctx, err := tracing.OpenTelemetryInit(context.Background())
 	if err != nil {
 		// openTelemetryInit can only fail if OpenTofu was run with an
@@ -102,6 +129,9 @@ func realMain() int {
 		log.Printf("[DEBUG] using %s %s", depMod.Path, depMod.Version)
 	}
 	log.Printf("[INFO] Go runtime version: %s", runtime.Version())
+	if fips140.Enabled() {
+		log.Printf("[WARNING] Go runtime FIPS 140-3 mode is enabled; OpenTofu is not supported in this configuration, which may cause undesirable behavior")
+	}
 	log.Printf("[INFO] CLI args: %#v", os.Args)
 	if experimentsAreAllowed() {
 		log.Printf("[INFO] This build of OpenTofu allows using experimental features")
@@ -169,25 +199,16 @@ func realMain() int {
 		log.Printf("[WARN] Cannot initialize remote host credentials manager: %s", err)
 		credsSrc = nil // must be an untyped nil for newServiceDiscovery to understand "no credentials available"
 	}
-	services := newServiceDiscovery(ctx, credsSrc)
+	services := newServiceDiscovery(ctx, config.RegistryProtocols, credsSrc)
 
 	modulePkgFetcher := remoteModulePackageFetcher(ctx, config.OCICredentialsPolicy)
 
-	providerSrc, diags := providerSource(config.ProviderInstallation, services, config.OCICredentialsPolicy)
-	if len(diags) > 0 {
-		Ui.Error("There are some problems with the provider_installation configuration:")
-		for _, diag := range diags {
-			earlyColor := &colorstring.Colorize{
-				Colors:  colorstring.DefaultColors,
-				Disable: true, // Disable color to be conservative until we know better
-				Reset:   true,
-			}
-			Ui.Error(format.Diagnostic(diag, nil, earlyColor, 78))
-		}
-		if diags.HasErrors() {
-			Ui.Error("As a result of the above problems, OpenTofu's provider installer may not behave as intended.\n\n")
-			// We continue to run anyway, because most commands don't do provider installation.
-		}
+	// Get the original working directory before any -chdir processing
+	originalWd, err := os.Getwd()
+	if err != nil {
+		// It would be very strange to end up here
+		Ui.Error(fmt.Sprintf("Failed to determine current working directory: %s", err))
+		return 1
 	}
 	providerDevOverrides := providerDevOverrides(config.ProviderInstallation)
 
@@ -207,13 +228,6 @@ func realMain() int {
 	binName := filepath.Base(os.Args[0])
 	args := os.Args[1:]
 
-	originalWd, err := os.Getwd()
-	if err != nil {
-		// It would be very strange to end up here
-		Ui.Error(fmt.Sprintf("Failed to determine current working directory: %s", err))
-		return 1
-	}
-
 	// The arguments can begin with a -chdir option to ask OpenTofu to switch
 	// to a different working directory for the rest of its work. If that
 	// option is present then extractChdirOption returns a trimmed args with that option removed.
@@ -230,6 +244,29 @@ func realMain() int {
 		}
 	}
 
+	providerSrc, diags := providerSource(ctx,
+		config.ProviderInstallation,
+		config.RegistryProtocols,
+		services,
+		config.OCICredentialsPolicy,
+		originalWd,
+	)
+	if len(diags) > 0 {
+		Ui.Error("There are some problems with the provider_installation configuration:")
+		for _, diag := range diags {
+			earlyColor := &colorstring.Colorize{
+				Colors:  colorstring.DefaultColors,
+				Disable: true, // Disable color to be conservative until we know better
+				Reset:   true,
+			}
+			Ui.Error(format.Diagnostic(diag, nil, earlyColor, 78))
+		}
+		if diags.HasErrors() {
+			Ui.Error("As a result of the above problems, OpenTofu's provider installer may not behave as intended.\n\n")
+			// We continue to run anyway, because most commands don't do provider installation.
+		}
+	}
+
 	// In tests, Commands may already be set to provide mock commands
 	if commands == nil {
 		// Commands get to hold on to the original working directory here,
@@ -266,8 +303,8 @@ func realMain() int {
 	}
 
 	// Prefix the args with any args from the EnvCLI targeting this command
-	suffix := strings.Replace(strings.Replace(
-		cliRunner.Subcommand(), "-", "_", -1), " ", "_", -1)
+	suffix := strings.ReplaceAll(strings.ReplaceAll(
+		cliRunner.Subcommand(), "-", "_"), " ", "_")
 	args, err = mergeEnvArgs(
 		fmt.Sprintf("%s_%s", EnvCLI, suffix), cliRunner.Subcommand(), args)
 	if err != nil {
