@@ -9,14 +9,16 @@ import (
 	"context"
 	"errors"
 	"fmt"
+	"strings"
 
 	"github.com/hashicorp/hcl/v2"
-	"github.com/hashicorp/hcl/v2/gohcl"
+	"github.com/hashicorp/hcl/v2/hclsyntax"
 	"github.com/opentofu/opentofu/internal/addrs"
 	"github.com/opentofu/opentofu/internal/configs"
 	"github.com/opentofu/opentofu/internal/encryption/config"
 	"github.com/opentofu/opentofu/internal/encryption/method"
 	"github.com/opentofu/opentofu/internal/encryption/registry"
+	"github.com/zclconf/go-cty/cty"
 )
 
 // setupMethod sets up a single method for encryption. It returns a list of diagnostics if the method is invalid.
@@ -43,37 +45,60 @@ func setupMethod(ctx context.Context, enc *config.EncryptionConfig, cfg config.M
 		}}
 	}
 
-	methodConfig := encryptionMethod.ConfigStruct()
+	var methodCtx method.EvalContext
+	methodCtx = method.EvalContext{ValueForExpression: func(expr hcl.Expression) (cty.Value, hcl.Diagnostics) {
+		var diags hcl.Diagnostics
 
-	deps, diags := gohcl.VariablesInBody(cfg.Body, methodConfig)
-	if diags.HasErrors() {
-		return nil, diags
-	}
+		deps := expr.Variables()
 
-	kpConfigs, refs, kpDiags := filterKeyProviderReferences(enc, deps)
-	diags = diags.Extend(kpDiags)
-	if diags.HasErrors() {
-		return nil, diags
-	}
+		kpConfigs, refs, kpDiags := filterKeyProviderReferences(enc, deps)
+		diags = diags.Extend(kpDiags)
+		if diags.HasErrors() {
+			return cty.NilVal, diags
+		}
 
-	hclCtx, kpDiags := setupKeyProviders(ctx, enc, kpConfigs, meta, reg, staticEval)
-	diags = diags.Extend(kpDiags)
-	if diags.HasErrors() {
-		return nil, diags
-	}
+		hclCtx, kpDiags := setupKeyProviders(ctx, enc, kpConfigs, meta, reg, staticEval)
+		diags = diags.Extend(kpDiags)
+		if diags.HasErrors() {
+			return cty.NilVal, diags
+		}
 
-	hclCtx, evalDiags := staticEval.EvalContextWithParent(ctx, hclCtx, configs.StaticIdentifier{
-		Module:    addrs.RootModule,
-		Subject:   fmt.Sprintf("encryption.method.%s.%s", cfg.Type, cfg.Name),
-		DeclRange: enc.DeclRange,
-	}, refs)
-	diags = diags.Extend(evalDiags)
-	if diags.HasErrors() {
-		return nil, diags
-	}
+		hclCtx, evalDiags := staticEval.EvalContextWithParent(ctx, hclCtx, configs.StaticIdentifier{
+			Module:    addrs.RootModule,
+			Subject:   fmt.Sprintf("encryption.method.%s.%s", cfg.Type, cfg.Name),
+			DeclRange: enc.DeclRange,
+		}, refs)
+		diags = diags.Extend(evalDiags)
+		if diags.HasErrors() {
+			return cty.NilVal, diags
+		}
 
-	methodDiags := gohcl.DecodeBody(cfg.Body, hclCtx, methodConfig)
-	diags = diags.Extend(methodDiags)
+		val, valDiags := expr.Value(hclCtx)
+		diags = diags.Extend(valDiags)
+		if diags.HasErrors() {
+			return cty.NilVal, diags
+		}
+
+		if val.Type() == cty.String {
+			// Try to be clever to see if it's a kp string that is actually a reference
+			// We might want a bool to opt-in to this functionality for JSON compat on specific fields
+
+			str := val.AsString()
+			if strings.HasPrefix(str, "key_provider.") {
+				traversal, travDiags := hclsyntax.ParseTraversalAbs([]byte(str), "", hcl.Pos{})
+				if !travDiags.HasErrors() {
+					// Call into the expr resolver again
+					val, valDiags = methodCtx.ValueForExpression(&hclsyntax.ScopeTraversalExpr{Traversal: traversal, SrcRange: expr.Range()})
+					diags = diags.Extend(valDiags)
+				}
+			}
+
+		}
+
+		return val, diags
+	}}
+
+	methodConfig, diags := encryptionMethod.DecodeConfig(methodCtx, cfg.Body)
 	if diags.HasErrors() {
 		return nil, diags
 	}
